package mathUtil;

/**
Abstract Function class used by Netwon Raphson
*/
public abstract class AbstractFunctionObject {

  //no of variables for each function
  int n;

  private static final boolean DEBUG=false;

  /**
     Constructor takes number of functions.
     */
  public AbstractFunctionObject(int n) {
    this.n = n;
  }//end of constructor

  /**
     evaluate the i-th function
     */
  public abstract double evaluate(int i,double val[]);

  /**
     evalauate the j-th derivative for the i-th function
     */
  public double derivative(int i,int j, double val[], double delta) {
    double new_val[] = new double[val.length];
    System.arraycopy(val,0,new_val,0,val.length);

    if (DEBUG) {
      System.out.println("i and j ");
      System.out.print(i);
      System.out.print(" ");
      System.out.println(j);
    }



    //pertrub the j-th entry by delta
    new_val[j]=new_val[j]+delta;


    if (DEBUG) {
      System.out.print("New and old values ");

      for(int k=0; k < n ; k++) {
	System.out.print("|");
	System.out.print(val[k]);
	System.out.print(" ");
	System.out.println(new_val[k]);
      }
    }

    double eval1 = evaluate(i,new_val);
    double eval2 = evaluate(i,val);
    
    if (DEBUG) {
      System.out.print("Eval1 Eval2 ");
      System.out.print(eval1);
      System.out.print(" ");
      System.out.println(eval2);
    }

    double diff = eval1-eval2;

    if (DEBUG) {
      System.out.print("Diff/Delta/return value ");
      System.out.print(diff);
      System.out.print(" ");
      System.out.print(delta);
      System.out.print(" ");
      System.out.println(diff/delta);
    }

    return(diff/delta);
  }//end of derivative

}//end of AbstractFunctionObject
